chapter \<open>Calculational reasoning\<close>

theory Calculation
imports Main
begin

section \<open>Overview\<close>

text \<open>
  \<^item> build transitive chains of forward-reasoning, based on flow of facts:
    primary \<^theory_text>\<open>this\<close> versus secondary \<^theory_text>\<open>calculation\<close>

  \<^item> facts are combined via rules from the context:
    command \<^theory_text>\<open>also\<close> and \<^theory_text>\<open>finally\<close>

  \<^item> rules may be given explicitly in special situations:
    command \<^theory_text>\<open>also (rules)\<close>

  \<^item> degenerate calculation without rules are also possible:
    command \<^theory_text>\<open>moreover\<close> and \<^theory_text>\<open>ultimately\<close>
\<close>


section \<open>Some special names in Isar proofs\<close>

text \<open>
  \<^theory_text>\<open>term ?thesis\<close>
    the conclusion of the current claim

  \<^theory_text>\<open>thm that\<close>
    the premises of the current claim

  \<^theory_text>\<open>thm this\<close>
    the current fact produced in the preceding text

  \<^theory_text>\<open>term \<dots>\<close>
    the argument of the last explicitly stated result
    (for infix application this is the right-hand side)
\<close>

notepad
begin
  fix x y :: 'a
  have "x = y" if "y = x"
  proof -
    term ?thesis
    thm that
    show ?thesis sorry
    term ?thesis  \<comment> \<open>static, unchanged!\<close>
  qed
  thm this
  term \<dots>
end

text \<open>
  Calculational reasoning maintains a fact \<^theory_text>\<open>calculation\<close> in the background.
  Certain language elements combine primary \<^theory_text>\<open>this\<close> with secondary
  \<^theory_text>\<open>calculation\<close>.
\<close>


section \<open>Transitive chains\<close>

text \<open>
  Idea: combine \<^theory_text>\<open>this\<close> and \<^theory_text>\<open>calculation\<close> via \<^theory_text>\<open>trans\<close> rules.
\<close>

thm
  trans
  less_trans
  less_le_trans

notepad
begin
  fix a b c d :: 'a

  txt \<open>plain bottom-up calculation\<close>
  have "a = b" sorry
  also
  have "b = c" sorry
  also
  have "c = d" sorry
  finally
  have "a = d" .

  txt \<open>variant with \<open>\<dots>\<close> abbreviation\<close>
  have "a = b" sorry
  also
  have "\<dots> = c" sorry
  also
  have "\<dots> = d" sorry
  finally
  have "a = d" .

  txt \<open>top-down version with explicit claim at the head\<close>
  have "a = d"
  proof -
    have "a = b" sorry
    also
    have "\<dots> = c" sorry
    also
    have "\<dots> = d" sorry
    finally
    show ?thesis .
  qed
next
  fix a b c d :: nat
  txt \<open>mixed inequalities; requires suitable base type\<close>

  have "a < b" sorry
  also
  have "b \<le> c" sorry
  also
  have "c = d" sorry
  finally
  have "a < d" .
end

print_trans_rules \<comment> \<open>rules declared in the library\<close>

text \<open>
  Notes:

  \<^item> The notion of \<^theory_text>\<open>trans\<close> rule is very general due to the flexibility of Pure
    rule composition.

  \<^item> User applications may declare there own rules, with some care about the
    operational details of higher-order unification.
\<close>


section \<open>Exercise\<close>

text \<open>
  \<triangleright> Prove some elementary algebraic identities derived from group axioms.
\<close>

notepad
begin
  txt \<open>hypothetical group axiomatization\<close>

  fix prod :: "'a \<Rightarrow> 'a \<Rightarrow> 'a"  (infixl "\<odot>" 70)
    and one :: "'a"
    and inverse :: "'a \<Rightarrow> 'a"
  assume assoc: "(x \<odot> y) \<odot> z = x \<odot> (y \<odot> z)"
    and left_one: "one \<odot> x = x"
    and left_inverse: "inverse x \<odot> x = one"
    for x y z

  txt \<open>some consequences\<close>

  have right_inverse: "x \<odot> inverse x = one" for x
  proof -
    txt \<open>
      "x \<odot> inverse x = one \<odot> (x \<odot> inverse x)"  (left_one)
      "\<dots> = one \<odot> x \<odot> inverse x"  (assoc)
      "\<dots> = inverse (inverse x) \<odot> inverse x \<odot> x \<odot> inverse x"  (left_inverse)
      "\<dots> = inverse (inverse x) \<odot> (inverse x \<odot> x) \<odot> inverse x"  (assoc)
      "\<dots> = inverse (inverse x) \<odot> one \<odot> inverse x"  (left_inverse)
      "\<dots> = inverse (inverse x) \<odot> (one \<odot> inverse x)"  (assoc)
      "\<dots> = inverse (inverse x) \<odot> inverse x"  (left_one)
      "\<dots> = one"  (left_inverse)
    \<close>
    show ?thesis sorry
  qed

  have right_one: "x \<odot> one = x" for x
  proof -
    txt \<open>
      "x \<odot> one = x \<odot> (inverse x \<odot> x)"  (left_inverse)
      "\<dots> = x \<odot> inverse x \<odot> x"  (assoc)
      "\<dots> = one \<odot> x"  (right_inverse)
      "\<dots> = x"  (left_one)
    \<close>
    show ?thesis sorry
  qed

  have one_equality: "one = e" if "e \<odot> x = x" for e x
  proof -
    show ?thesis using that sorry
  qed

  have inverse_equality: "inverse x = x'" if "x' \<odot> x = one" for x x'
  proof -
    show ?thesis using that sorry
  qed

end

end
