chapter \<open>Isabelle documents\<close>

theory Document
imports Main
begin

section \<open>Document structure\<close>

text \<open>
  \<^item> outermost formal theory structure \<^theory_text>\<open>theory \<dots> begin \<dots> end\<close>
    with formal specification and proofs in the body

  \<^item> commands for document structure:
    \<^theory_text>\<open>chapter\<close>, \<^theory_text>\<open>section\<close>, \<^theory_text>\<open>subsection\<close> etc.

  \<^item> commands for document content (paragraphs of text):
    \<^theory_text>\<open>text\<close>, \<^theory_text>\<open>txt\<close>, \<^theory_text>\<open>text_raw\<close>

  \<^item> side-comments (snippets of text):
    \<comment> \<open>comment\<close>
\<close>


section \<open>Quotation and anti-quotation\<close>

text \<open>
  \<^item> outermost theory text is formal

  \<^item> informal document text is quoted inside the theory

  \<^item> formal entities are embedded as antiquotations inside document text

    \<open>
      \<^name>\<open>argument content\<close>
      @{name arguments \<dots>}
    \<close>

    \<open>uninterpreted formal text (no syntax)\<close>
    \<^theory_text>\<open>uninterpreted formal "text" (outer "syntax")\<close>

    @{typ 'a}
    @{term x}
    @{prop "x = x"}
    @{thm refl}
\<close>


subsection \<open>Example\<close>

notepad  \<comment> \<open>begin formal proof context\<close>
begin
  fix x :: nat
    \<comment> \<open>let @{term x} be arbitrary but fixed\<close>
    \<comment> \<open>\<dots> and of type @{typ nat}\<close>
  text \<open>
    This is a paragraph within the proof context.
    We are going to prove something trivial about \<open>x\<close>:
  \<close>
  have "x = x" by (rule refl)
end


section \<open>Exercise\<close>

text \<open>
  \<triangleright> Turn some of your finished theories into a printed document.

  See also @{doc system}, chapter 3.
\<close>

end